@@ -47,6 +47,21 @@ fn main() {
     }
 }
 
+/// Table for defining the aliases which come builtin in `Cargo`.
+/// The contents are structured as: `(alias, aliased_command, description)`.
+const BUILTIN_ALIASES: [(&str, &str, &str); 4] = [
+    ("b", "build", "alias: build"),
+    ("c", "check", "alias: check"),
+    ("r", "run", "alias: run"),
+    ("t", "test", "alias: test"),
+];
+
+/// Function which contains the list of all of the builtin aliases and it's
+/// corresponding execs represented as &str.
+fn builtin_aliases_execs(cmd: &str) -> Option<&(&str, &str, &str)> {
+    BUILTIN_ALIASES.iter().find(|alias| alias.0 == cmd)
+}
+
 fn aliased_command(config: &Config, command: &str) -> CargoResult<Option<Vec<String>>> {
     let alias_name = format!("alias.{}", command);
     let user_alias = match config.get_string(&alias_name) {
@@ -60,12 +75,10 @@ fn aliased_command(config: &Config, command: &str) -> CargoResult<Option<Vec<Str
         Ok(None) => None,
         Err(_) => config.get::<Option<Vec<String>>>(&alias_name)?,
     };
-    let result = user_alias.or_else(|| match command {
-        "b" => Some(vec!["build".to_string()]),
-        "c" => Some(vec!["check".to_string()]),
-        "r" => Some(vec!["run".to_string()]),
-        "t" => Some(vec!["test".to_string()]),
-        _ => None,
+
+    let result = user_alias.or_else(|| match builtin_aliases_execs(command) {
+        Some(command_str) => Some(vec![command_str.1.to_string()]),
+        None => None,
     });
     Ok(result)
 }
@@ -106,6 +119,15 @@ fn list_commands(config: &Config) -> BTreeSet<CommandInfo> {
         });
     }
 
+    // Add the builtin_aliases and them descriptions to the
+    // `commands` `BTreeSet`.
+    for command in &BUILTIN_ALIASES {
+        commands.insert(CommandInfo::BuiltIn {
+            name: command.0.to_string(),
+            about: Some(command.2.to_string()),
+        });
+    }
+
     commands
 }
 
