@@ -25,6 +25,8 @@ pub enum EitherManifest {
 }
 
 /// Contains all the information about a package, as loaded from a `Cargo.toml`.
+///
+/// This is deserialized using the [`TomlManifest`] type.
 #[derive(Clone, Debug)]
 pub struct Manifest {
     summary: Summary,
@@ -42,7 +44,7 @@ pub struct Manifest {
     patch: HashMap<Url, Vec<Dependency>>,
     workspace: WorkspaceConfig,
     original: Rc<TomlManifest>,
-    features: Features,
+    unstable_features: Features,
     edition: Edition,
     im_a_teapot: Option<bool>,
     default_run: Option<String>,
@@ -220,10 +222,7 @@ impl TargetSourcePath {
     }
 
     pub fn is_path(&self) -> bool {
-        match self {
-            TargetSourcePath::Path(_) => true,
-            _ => false,
-        }
+        matches!(self, TargetSourcePath::Path(_))
     }
 }
 
@@ -262,7 +261,12 @@ struct SerializedTarget<'a> {
     edition: &'a str,
     #[serde(rename = "required-features", skip_serializing_if = "Option::is_none")]
     required_features: Option<Vec<&'a str>>,
+    /// Whether docs should be built for the target via `cargo doc`
+    /// See https://doc.rust-lang.org/cargo/commands/cargo-doc.html#target-selection
+    doc: bool,
     doctest: bool,
+    /// Whether tests should be run for the target (`test` field in `Cargo.toml`)
+    test: bool,
 }
 
 impl ser::Serialize for Target {
@@ -282,7 +286,9 @@ impl ser::Serialize for Target {
             required_features: self
                 .required_features()
                 .map(|rf| rf.iter().map(|s| &**s).collect()),
+            doc: self.documented(),
             doctest: self.doctested() && self.doctestable(),
+            test: self.tested(),
         }
         .serialize(s)
     }
@@ -371,7 +377,7 @@ impl Manifest {
         replace: Vec<(PackageIdSpec, Dependency)>,
         patch: HashMap<Url, Vec<Dependency>>,
         workspace: WorkspaceConfig,
-        features: Features,
+        unstable_features: Features,
         edition: Edition,
         im_a_teapot: Option<bool>,
         default_run: Option<String>,
@@ -393,7 +399,7 @@ impl Manifest {
             replace,
             patch,
             workspace,
-            features,
+            unstable_features,
             edition,
             original,
             im_a_teapot,
@@ -431,6 +437,10 @@ impl Manifest {
     pub fn targets(&self) -> &[Target] {
         &self.targets
     }
+    // It is used by cargo-c, please do not remove it
+    pub fn targets_mut(&mut self) -> &mut [Target] {
+        &mut self.targets
+    }
     pub fn version(&self) -> &Version {
         self.package_id().version()
     }
@@ -463,8 +473,9 @@ impl Manifest {
         &self.workspace
     }
 
-    pub fn features(&self) -> &Features {
-        &self.features
+    /// Unstable, nightly features that are enabled in this manifest.
+    pub fn unstable_features(&self) -> &Features {
+        &self.unstable_features
     }
 
     /// The style of resolver behavior to use, declared with the `resolver` field.
@@ -483,7 +494,7 @@ impl Manifest {
 
     pub fn feature_gate(&self) -> CargoResult<()> {
         if self.im_a_teapot.is_some() {
-            self.features
+            self.unstable_features
                 .require(Feature::test_dummy_unstable())
                 .chain_err(|| {
                     anyhow::format_err!(
@@ -574,7 +585,7 @@ impl VirtualManifest {
         &self.warnings
     }
 
-    pub fn features(&self) -> &Features {
+    pub fn unstable_features(&self) -> &Features {
         &self.features
     }
 
@@ -770,10 +781,7 @@ impl Target {
     }
 
     pub fn is_lib(&self) -> bool {
-        match self.kind() {
-            TargetKind::Lib(_) => true,
-            _ => false,
-        }
+        matches!(self.kind(), TargetKind::Lib(_))
     }
 
     pub fn is_dylib(&self) -> bool {
@@ -806,10 +814,10 @@ impl Target {
     }
 
     pub fn is_example(&self) -> bool {
-        match self.kind() {
-            TargetKind::ExampleBin | TargetKind::ExampleLib(..) => true,
-            _ => false,
-        }
+        matches!(
+            self.kind(),
+            TargetKind::ExampleBin | TargetKind::ExampleLib(..)
+        )
     }
 
     /// Returns `true` if it is a binary or executable example.
@@ -821,10 +829,7 @@ impl Target {
     /// Returns `true` if it is an executable example.
     pub fn is_exe_example(&self) -> bool {
         // Needed for --all-examples in contexts where only runnable examples make sense
-        match self.kind() {
-            TargetKind::ExampleBin => true,
-            _ => false,
-        }
+        matches!(self.kind(), TargetKind::ExampleBin)
     }
 
     pub fn is_test(&self) -> bool {
