@@ -296,13 +296,10 @@ impl Profiles {
             // `--release` and `--debug` predicates, and convert back from
             // ProfileKind::Custom instantiation.
 
-            let release = match self.requested_profile.as_str() {
-                "release" | "bench" => true,
-                _ => false,
-            };
+            let release = matches!(self.requested_profile.as_str(), "release" | "bench");
 
             match mode {
-                CompileMode::Test | CompileMode::Bench => {
+                CompileMode::Test | CompileMode::Bench | CompileMode::Doctest => {
                     if release {
                         (
                             InternedString::new("bench"),
@@ -315,10 +312,7 @@ impl Profiles {
                         )
                     }
                 }
-                CompileMode::Build
-                | CompileMode::Check { .. }
-                | CompileMode::Doctest
-                | CompileMode::RunCustomBuild => {
+                CompileMode::Build | CompileMode::Check { .. } | CompileMode::RunCustomBuild => {
                     // Note: `RunCustomBuild` doesn't normally use this code path.
                     // `build_unit_profiles` normally ensures that it selects the
                     // ancestor's profile. However, `cargo clean -p` can hit this
@@ -470,8 +464,31 @@ impl ProfileMaker {
         unit_for: UnitFor,
     ) -> Profile {
         let mut profile = self.default;
+
+        // First apply profile-specific settings, things like
+        // `[profile.release]`
         if let Some(toml) = &self.toml {
             merge_profile(&mut profile, toml);
+        }
+
+        // Next start overriding those settings. First comes build dependencies
+        // which default to opt-level 0...
+        if unit_for.is_for_host() {
+            // For-host units are things like procedural macros, build scripts, and
+            // their dependencies. For these units most projects simply want them
+            // to compile quickly and the runtime doesn't matter too much since
+            // they tend to process very little data. For this reason we default
+            // them to a "compile as quickly as possible" mode which for now means
+            // basically turning down the optimization level and avoid limiting
+            // codegen units. This ensures that we spend little time optimizing as
+            // well as enabling parallelism by not constraining codegen units.
+            profile.opt_level = InternedString::new("0");
+            profile.codegen_units = None;
+        }
+        // ... and next comes any other sorts of overrides specified in
+        // profiles, such as `[profile.release.build-override]` or
+        // `[profile.release.package.foo]`
+        if let Some(toml) = &self.toml {
             merge_toml_overrides(pkg_id, is_member, unit_for, &mut profile, toml);
         }
         profile
@@ -487,7 +504,7 @@ fn merge_toml_overrides(
     toml: &TomlProfile,
 ) {
     if unit_for.is_for_host() {
-        if let Some(ref build_override) = toml.build_override {
+        if let Some(build_override) = &toml.build_override {
             merge_profile(profile, build_override);
         }
     }
@@ -950,6 +967,16 @@ impl UnitFor {
         }
     }
 
+    /// This is a special case for unit tests of a proc-macro.
+    ///
+    /// Proc-macro unit tests are forced to be run on the host.
+    pub fn new_host_test(config: &Config) -> UnitFor {
+        let mut unit_for = UnitFor::new_test(config);
+        unit_for.host = true;
+        unit_for.host_features = true;
+        unit_for
+    }
+
     /// Returns a new copy based on `for_host` setting.
     ///
     /// When `for_host` is true, this clears `panic_abort_ok` in a sticky
