@@ -57,16 +57,24 @@ type ActivateMap = HashMap<(PackageId, bool), BTreeSet<InternedString>>;
 /// Set of all activated features for all packages in the resolve graph.
 pub struct ResolvedFeatures {
     activated_features: ActivateMap,
+    /// Optional dependencies that should be built.
+    ///
+    /// The value is the `name_in_toml` of the dependencies.
+    activated_dependencies: ActivateMap,
     /// This is only here for legacy support when `-Zfeatures` is not enabled.
-    legacy: Option<HashMap<PackageId, Vec<InternedString>>>,
+    ///
+    /// This is the set of features enabled for each package.
+    legacy_features: Option<HashMap<PackageId, Vec<InternedString>>>,
+    /// This is only here for legacy support when `-Zfeatures` is not enabled.
+    ///
+    /// This is the set of optional dependencies enabled for each package.
+    legacy_dependencies: Option<HashMap<PackageId, HashSet<InternedString>>>,
     opts: FeatureOpts,
 }
 
 /// Options for how the feature resolver works.
 #[derive(Default)]
 struct FeatureOpts {
-    /// -Zpackage-features, changes behavior of feature flags in a workspace.
-    package_features: bool,
     /// -Zfeatures is enabled, use new resolver.
     new_resolver: bool,
     /// Build deps and proc-macros will not share share features with other dep kinds.
@@ -125,7 +133,6 @@ impl FeatureOpts {
     ) -> CargoResult<FeatureOpts> {
         let mut opts = FeatureOpts::default();
         let unstable_flags = ws.config().cli_unstable();
-        opts.package_features = unstable_flags.package_features;
         let mut enable = |feat_opts: &Vec<String>| {
             opts.new_resolver = true;
             for opt in feat_opts {
@@ -170,6 +177,10 @@ impl FeatureOpts {
         if let ForceAllTargets::Yes = force_all_targets {
             opts.ignore_inactive_targets = false;
         }
+        if unstable_flags.weak_dep_features {
+            // Force this ON because it only works with the new resolver.
+            opts.new_resolver = true;
+        }
         Ok(opts)
     }
 }
@@ -227,6 +238,30 @@ impl ResolvedFeatures {
             .expect("activated_features for invalid package")
     }
 
+    /// Returns if the given dependency should be included.
+    ///
+    /// This handles dependencies disabled via `cfg` expressions and optional
+    /// dependencies which are not enabled.
+    pub fn is_dep_activated(
+        &self,
+        pkg_id: PackageId,
+        features_for: FeaturesFor,
+        dep_name: InternedString,
+    ) -> bool {
+        if let Some(legacy) = &self.legacy_dependencies {
+            legacy
+                .get(&pkg_id)
+                .map(|deps| deps.contains(&dep_name))
+                .unwrap_or(false)
+        } else {
+            let is_build = self.opts.decouple_host_deps && features_for == FeaturesFor::HostDep;
+            self.activated_dependencies
+                .get(&(pkg_id, is_build))
+                .map(|deps| deps.contains(&dep_name))
+                .unwrap_or(false)
+        }
+    }
+
     /// Variant of `activated_features` that returns `None` if this is
     /// not a valid pkg_id/is_build combination. Used in places which do
     /// not know which packages are activated (like `cargo clean`).
@@ -243,7 +278,7 @@ impl ResolvedFeatures {
         pkg_id: PackageId,
         features_for: FeaturesFor,
     ) -> CargoResult<Vec<InternedString>> {
-        if let Some(legacy) = &self.legacy {
+        if let Some(legacy) = &self.legacy_features {
             Ok(legacy.get(&pkg_id).map_or_else(Vec::new, |v| v.clone()))
         } else {
             let is_build = self.opts.decouple_host_deps && features_for == FeaturesFor::HostDep;
@@ -267,9 +302,28 @@ pub struct FeatureResolver<'a, 'cfg> {
     opts: FeatureOpts,
     /// Map of features activated for each package.
     activated_features: ActivateMap,
+    /// Map of optional dependencies activated for each package.
+    activated_dependencies: ActivateMap,
     /// Keeps track of which packages have had its dependencies processed.
     /// Used to avoid cycles, and to speed up processing.
     processed_deps: HashSet<(PackageId, bool)>,
+    /// If this is `true`, then `for_host` needs to be tracked while
+    /// traversing the graph.
+    ///
+    /// This is only here to avoid calling `is_proc_macro` when all feature
+    /// options are disabled (because `is_proc_macro` can trigger downloads).
+    /// This has to be separate from `FeatureOpts.decouple_host_deps` because
+    /// `for_host` tracking is also needed for `itarget` to work properly.
+    track_for_host: bool,
+    /// `dep_name?/feat_name` features that will be activated if `dep_name` is
+    /// ever activated.
+    ///
+    /// The key is the `(package, for_host, dep_name)` of the package whose
+    /// dependency will trigger the addition of new features. The value is the
+    /// set of `(feature, dep_prefix)` features to activate (`dep_prefix` is a
+    /// bool that indicates if `dep:` prefix was used).
+    deferred_weak_dependencies:
+        HashMap<(PackageId, bool, InternedString), HashSet<(InternedString, bool)>>,
 }
 
 impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
@@ -294,10 +348,13 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
             // Legacy mode.
             return Ok(ResolvedFeatures {
                 activated_features: HashMap::new(),
-                legacy: Some(resolve.features_clone()),
+                activated_dependencies: HashMap::new(),
+                legacy_features: Some(resolve.features_clone()),
+                legacy_dependencies: Some(compute_legacy_deps(resolve)),
                 opts,
             });
         }
+        let track_for_host = opts.decouple_host_deps || opts.ignore_inactive_targets;
         let mut r = FeatureResolver {
             ws,
             target_data,
@@ -306,7 +363,10 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
             package_set,
             opts,
             activated_features: HashMap::new(),
+            activated_dependencies: HashMap::new(),
             processed_deps: HashSet::new(),
+            track_for_host,
+            deferred_weak_dependencies: HashMap::new(),
         };
         r.do_resolve(specs, requested_features)?;
         log::debug!("features={:#?}", r.activated_features);
@@ -315,7 +375,9 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
         }
         Ok(ResolvedFeatures {
             activated_features: r.activated_features,
-            legacy: None,
+            activated_dependencies: r.activated_dependencies,
+            legacy_features: None,
+            legacy_dependencies: None,
             opts: r.opts,
         })
     }
@@ -329,8 +391,8 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
         let member_features = self.ws.members_with_features(specs, requested_features)?;
         for (member, requested_features) in &member_features {
             let fvs = self.fvs_from_requested(member.package_id(), requested_features);
-            let for_host = self.is_proc_macro(member.package_id());
-            self.activate_pkg(member.package_id(), &fvs, for_host)?;
+            let for_host = self.track_for_host && self.is_proc_macro(member.package_id());
+            self.activate_pkg(member.package_id(), for_host, &fvs)?;
             if for_host {
                 // Also activate without for_host. This is needed if the
                 // proc-macro includes other targets (like binaries or tests),
@@ -339,7 +401,7 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
                 // `--workspace`), this forces feature unification with normal
                 // dependencies. This is part of the bigger problem where
                 // features depend on which packages are built.
-                self.activate_pkg(member.package_id(), &fvs, false)?;
+                self.activate_pkg(member.package_id(), false, &fvs)?;
             }
         }
         Ok(())
@@ -348,9 +410,10 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
     fn activate_pkg(
         &mut self,
         pkg_id: PackageId,
-        fvs: &[FeatureValue],
         for_host: bool,
+        fvs: &[FeatureValue],
     ) -> CargoResult<()> {
+        log::trace!("activate_pkg {} {}", pkg_id.name(), for_host);
         // Add an empty entry to ensure everything is covered. This is intended for
         // finding bugs where the resolver missed something it should have visited.
         // Remove this in the future if `activated_features` uses an empty default.
@@ -358,7 +421,7 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
             .entry((pkg_id, self.opts.decouple_host_deps && for_host))
             .or_insert_with(BTreeSet::new);
         for fv in fvs {
-            self.activate_fv(pkg_id, fv, for_host)?;
+            self.activate_fv(pkg_id, for_host, fv)?;
         }
         if !self.processed_deps.insert((pkg_id, for_host)) {
             // Already processed dependencies. There's no need to process them
@@ -371,7 +434,7 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
             // For example, consider we've already processed our dependencies,
             // and another package comes along and enables one of our optional
             // dependencies, it will do so immediately in the
-            // `FeatureValue::CrateFeature` branch, and then immediately
+            // `FeatureValue::DepFeature` branch, and then immediately
             // recurse into that optional dependency. This also holds true for
             // features that enable other features.
             return Ok(());
@@ -385,7 +448,7 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
                 }
                 // Recurse into the dependency.
                 let fvs = self.fvs_from_dependency(dep_pkg_id, dep);
-                self.activate_pkg(dep_pkg_id, &fvs, dep_for_host)?;
+                self.activate_pkg(dep_pkg_id, dep_for_host, &fvs)?;
             }
         }
         Ok(())
@@ -395,45 +458,31 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
     fn activate_fv(
         &mut self,
         pkg_id: PackageId,
-        fv: &FeatureValue,
         for_host: bool,
+        fv: &FeatureValue,
     ) -> CargoResult<()> {
+        log::trace!("activate_fv {} {} {}", pkg_id.name(), for_host, fv);
         match fv {
             FeatureValue::Feature(f) => {
-                self.activate_rec(pkg_id, *f, for_host)?;
+                self.activate_rec(pkg_id, for_host, *f)?;
             }
-            FeatureValue::Crate(dep_name) => {
-                // Activate the feature name on self.
-                self.activate_rec(pkg_id, *dep_name, for_host)?;
-                // Activate the optional dep.
-                for (dep_pkg_id, deps) in self.deps(pkg_id, for_host) {
-                    for (dep, dep_for_host) in deps {
-                        if dep.name_in_toml() != *dep_name {
-                            continue;
-                        }
-                        let fvs = self.fvs_from_dependency(dep_pkg_id, dep);
-                        self.activate_pkg(dep_pkg_id, &fvs, dep_for_host)?;
-                    }
-                }
+            FeatureValue::Dep { dep_name } => {
+                self.activate_dependency(pkg_id, for_host, *dep_name)?;
             }
-            FeatureValue::CrateFeature(dep_name, dep_feature) => {
-                // Activate a feature within a dependency.
-                for (dep_pkg_id, deps) in self.deps(pkg_id, for_host) {
-                    for (dep, dep_for_host) in deps {
-                        if dep.name_in_toml() != *dep_name {
-                            continue;
-                        }
-                        if dep.is_optional() {
-                            // Activate the crate on self.
-                            let fv = FeatureValue::Crate(*dep_name);
-                            self.activate_fv(pkg_id, &fv, for_host)?;
-                        }
-                        // Activate the feature on the dependency.
-                        let summary = self.resolve.summary(dep_pkg_id);
-                        let fv = FeatureValue::new(*dep_feature, summary);
-                        self.activate_fv(dep_pkg_id, &fv, dep_for_host)?;
-                    }
-                }
+            FeatureValue::DepFeature {
+                dep_name,
+                dep_feature,
+                dep_prefix,
+                weak,
+            } => {
+                self.activate_dep_feature(
+                    pkg_id,
+                    for_host,
+                    *dep_name,
+                    *dep_feature,
+                    *dep_prefix,
+                    *weak,
+                )?;
             }
         }
         Ok(())
@@ -444,9 +493,15 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
     fn activate_rec(
         &mut self,
         pkg_id: PackageId,
-        feature_to_enable: InternedString,
         for_host: bool,
+        feature_to_enable: InternedString,
     ) -> CargoResult<()> {
+        log::trace!(
+            "activate_rec {} {} feat={}",
+            pkg_id.name(),
+            for_host,
+            feature_to_enable
+        );
         let enabled = self
             .activated_features
             .entry((pkg_id, self.opts.decouple_host_deps && for_host))
@@ -472,7 +527,111 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
             }
         };
         for fv in fvs {
-            self.activate_fv(pkg_id, fv, for_host)?;
+            self.activate_fv(pkg_id, for_host, fv)?;
+        }
+        Ok(())
+    }
+
+    /// Activate a dependency (`dep:dep_name` syntax).
+    fn activate_dependency(
+        &mut self,
+        pkg_id: PackageId,
+        for_host: bool,
+        dep_name: InternedString,
+    ) -> CargoResult<()> {
+        // Mark this dependency as activated.
+        let save_for_host = self.opts.decouple_host_deps && for_host;
+        self.activated_dependencies
+            .entry((pkg_id, save_for_host))
+            .or_default()
+            .insert(dep_name);
+        // Check for any deferred features.
+        let to_enable = self
+            .deferred_weak_dependencies
+            .remove(&(pkg_id, for_host, dep_name));
+        // Activate the optional dep.
+        for (dep_pkg_id, deps) in self.deps(pkg_id, for_host) {
+            for (dep, dep_for_host) in deps {
+                if dep.name_in_toml() != dep_name {
+                    continue;
+                }
+                if let Some(to_enable) = &to_enable {
+                    for (dep_feature, dep_prefix) in to_enable {
+                        log::trace!(
+                            "activate deferred {} {} -> {}/{}",
+                            pkg_id.name(),
+                            for_host,
+                            dep_name,
+                            dep_feature
+                        );
+                        if !dep_prefix {
+                            self.activate_rec(pkg_id, for_host, dep_name)?;
+                        }
+                        let fv = FeatureValue::new(*dep_feature);
+                        self.activate_fv(dep_pkg_id, dep_for_host, &fv)?;
+                    }
+                }
+                let fvs = self.fvs_from_dependency(dep_pkg_id, dep);
+                self.activate_pkg(dep_pkg_id, dep_for_host, &fvs)?;
+            }
+        }
+        Ok(())
+    }
+
+    /// Activate a feature within a dependency (`dep_name/feat_name` syntax).
+    fn activate_dep_feature(
+        &mut self,
+        pkg_id: PackageId,
+        for_host: bool,
+        dep_name: InternedString,
+        dep_feature: InternedString,
+        dep_prefix: bool,
+        weak: bool,
+    ) -> CargoResult<()> {
+        for (dep_pkg_id, deps) in self.deps(pkg_id, for_host) {
+            for (dep, dep_for_host) in deps {
+                if dep.name_in_toml() != dep_name {
+                    continue;
+                }
+                if dep.is_optional() {
+                    let save_for_host = self.opts.decouple_host_deps && for_host;
+                    if weak
+                        && !self
+                            .activated_dependencies
+                            .get(&(pkg_id, save_for_host))
+                            .map(|deps| deps.contains(&dep_name))
+                            .unwrap_or(false)
+                    {
+                        // This is weak, but not yet activated. Defer in case
+                        // something comes along later and enables it.
+                        log::trace!(
+                            "deferring feature {} {} -> {}/{}",
+                            pkg_id.name(),
+                            for_host,
+                            dep_name,
+                            dep_feature
+                        );
+                        self.deferred_weak_dependencies
+                            .entry((pkg_id, for_host, dep_name))
+                            .or_default()
+                            .insert((dep_feature, dep_prefix));
+                        continue;
+                    }
+
+                    // Activate the dependency on self.
+                    let fv = FeatureValue::Dep { dep_name };
+                    self.activate_fv(pkg_id, for_host, &fv)?;
+                    if !dep_prefix {
+                        // To retain compatibility with old behavior,
+                        // this also enables a feature of the same
+                        // name.
+                        self.activate_rec(pkg_id, for_host, dep_name)?;
+                    }
+                }
+                // Activate the feature on the dependency.
+                let fv = FeatureValue::new(dep_feature);
+                self.activate_fv(dep_pkg_id, dep_for_host, &fv)?;
+            }
         }
         Ok(())
     }
@@ -484,7 +643,7 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
         let mut result: Vec<FeatureValue> = dep
             .features()
             .iter()
-            .map(|f| FeatureValue::new(*f, summary))
+            .map(|f| FeatureValue::new(*f))
             .collect();
         let default = InternedString::new("default");
         if dep.uses_default_features() && feature_map.contains_key(&default) {
@@ -502,28 +661,16 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
         let summary = self.resolve.summary(pkg_id);
         let feature_map = summary.features();
         if requested_features.all_features {
-            let mut fvs: Vec<FeatureValue> = feature_map
+            feature_map
                 .keys()
                 .map(|k| FeatureValue::Feature(*k))
-                .collect();
-            // Add optional deps.
-            // Top-level requested features can never apply to
-            // build-dependencies, so for_host is `false` here.
-            for (_dep_pkg_id, deps) in self.deps(pkg_id, false) {
-                for (dep, _dep_for_host) in deps {
-                    if dep.is_optional() {
-                        // This may result in duplicates, but that should be ok.
-                        fvs.push(FeatureValue::Crate(dep.name_in_toml()));
-                    }
-                }
-            }
-            fvs
+                .collect()
         } else {
             let mut result: Vec<FeatureValue> = requested_features
                 .features
                 .as_ref()
                 .iter()
-                .map(|f| FeatureValue::new(*f, summary))
+                .map(|f| FeatureValue::new(*f))
                 .collect();
             let default = InternedString::new("default");
             if requested_features.uses_default_features && feature_map.contains_key(&default) {
@@ -557,7 +704,6 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
         self.resolve
             .deps(pkg_id)
             .map(|(dep_id, deps)| {
-                let is_proc_macro = self.is_proc_macro(dep_id);
                 let deps = deps
                     .iter()
                     .filter(|dep| {
@@ -573,7 +719,8 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
                         true
                     })
                     .map(|dep| {
-                        let dep_for_host = for_host || dep.is_build() || is_proc_macro;
+                        let dep_for_host = self.track_for_host
+                            && (for_host || dep.is_build() || self.is_proc_macro(dep_id));
                         (dep, dep_for_host)
                     })
                     .collect::<Vec<_>>();
@@ -612,3 +759,19 @@ impl<'a, 'cfg> FeatureResolver<'a, 'cfg> {
             .proc_macro()
     }
 }
+
+/// Computes a map of PackageId to the set of optional dependencies that are
+/// enabled for that dep (when the new resolver is not enabled).
+fn compute_legacy_deps(resolve: &Resolve) -> HashMap<PackageId, HashSet<InternedString>> {
+    let mut result: HashMap<PackageId, HashSet<InternedString>> = HashMap::new();
+    for pkg_id in resolve.iter() {
+        for (_dep_id, deps) in resolve.deps(pkg_id) {
+            for dep in deps {
+                if dep.is_optional() {
+                    result.entry(pkg_id).or_default().insert(dep.name_in_toml());
+                }
+            }
+        }
+    }
+    result
+}
