@@ -1,3 +1,10 @@
+use crate::core::PackageId;
+use crate::sources::DirectorySource;
+use crate::sources::{GitSource, PathSource, RegistrySource, CRATES_IO_INDEX};
+use crate::util::{CanonicalUrl, CargoResult, Config, IntoUrl};
+use log::trace;
+use serde::de;
+use serde::ser;
 use std::cmp::{self, Ordering};
 use std::collections::HashSet;
 use std::fmt::{self, Formatter};
@@ -5,19 +12,10 @@ use std::hash::{self, Hash};
 use std::path::Path;
 use std::ptr;
 use std::sync::Mutex;
-
-use log::trace;
-use serde::de;
-use serde::ser;
 use url::Url;
 
-use crate::core::PackageId;
-use crate::sources::DirectorySource;
-use crate::sources::{GitSource, PathSource, RegistrySource, CRATES_IO_INDEX};
-use crate::util::{CanonicalUrl, CargoResult, Config, IntoUrl};
-
 lazy_static::lazy_static! {
-    static ref SOURCE_ID_CACHE: Mutex<HashSet<&'static SourceIdInner>> = Mutex::new(HashSet::new());
+    static ref SOURCE_ID_CACHE: Mutex<HashSet<&'static SourceIdInner>> = Default::default();
 }
 
 /// Unique identifier for a source of packages.
@@ -63,10 +61,12 @@ enum SourceKind {
 pub enum GitReference {
     /// From a tag.
     Tag(String),
-    /// From the HEAD of a branch.
+    /// From a branch.
     Branch(String),
     /// From a specific revision.
     Rev(String),
+    /// The default branch of the repository, the reference named `HEAD`.
+    DefaultBranch,
 }
 
 impl SourceId {
@@ -114,7 +114,7 @@ impl SourceId {
         match kind {
             "git" => {
                 let mut url = url.into_url()?;
-                let mut reference = GitReference::Branch("master".to_string());
+                let mut reference = GitReference::DefaultBranch;
                 for (k, v) in url.query_pairs() {
                     match &k[..] {
                         // Map older 'ref' to branch.
@@ -144,8 +144,8 @@ impl SourceId {
     }
 
     /// A view of the `SourceId` that can be `Display`ed as a URL.
-    pub fn into_url(&self) -> SourceIdIntoUrl<'_> {
-        SourceIdIntoUrl {
+    pub fn as_url(&self) -> SourceIdAsUrl<'_> {
+        SourceIdAsUrl {
             inner: &*self.inner,
         }
     }
@@ -239,10 +239,10 @@ impl SourceId {
 
     /// Returns `true` if this source is from a registry (either local or not).
     pub fn is_registry(self) -> bool {
-        match self.inner.kind {
-            SourceKind::Registry | SourceKind::LocalRegistry => true,
-            _ => false,
-        }
+        matches!(
+            self.inner.kind,
+            SourceKind::Registry | SourceKind::LocalRegistry
+        )
     }
 
     /// Returns `true` if this source is a "remote" registry.
@@ -250,18 +250,12 @@ impl SourceId {
     /// "remote" may also mean a file URL to a git index, so it is not
     /// necessarily "remote". This just means it is not `local-registry`.
     pub fn is_remote_registry(self) -> bool {
-        match self.inner.kind {
-            SourceKind::Registry => true,
-            _ => false,
-        }
+        matches!(self.inner.kind, SourceKind::Registry)
     }
 
     /// Returns `true` if this source from a Git repository.
     pub fn is_git(self) -> bool {
-        match self.inner.kind {
-            SourceKind::Git(_) => true,
-            _ => false,
-        }
+        matches!(self.inner.kind, SourceKind::Git(_))
     }
 
     /// Creates an implementation of `Source` corresponding to this ID.
@@ -367,15 +361,79 @@ impl SourceId {
     }
 }
 
+impl PartialEq for SourceId {
+    fn eq(&self, other: &SourceId) -> bool {
+        self.cmp(other) == Ordering::Equal
+    }
+}
+
 impl PartialOrd for SourceId {
     fn partial_cmp(&self, other: &SourceId) -> Option<Ordering> {
         Some(self.cmp(other))
     }
 }
 
+// Custom comparison defined as canonical URL equality for git sources and URL
+// equality for other sources, ignoring the `precise` and `name` fields.
 impl Ord for SourceId {
     fn cmp(&self, other: &SourceId) -> Ordering {
-        self.inner.cmp(other.inner)
+        // If our interior pointers are to the exact same `SourceIdInner` then
+        // we're guaranteed to be equal.
+        if ptr::eq(self.inner, other.inner) {
+            return Ordering::Equal;
+        }
+
+        // Sort first based on `kind`, deferring to the URL comparison below if
+        // the kinds are equal.
+        match (&self.inner.kind, &other.inner.kind) {
+            (SourceKind::Path, SourceKind::Path) => {}
+            (SourceKind::Path, _) => return Ordering::Less,
+            (_, SourceKind::Path) => return Ordering::Greater,
+
+            (SourceKind::Registry, SourceKind::Registry) => {}
+            (SourceKind::Registry, _) => return Ordering::Less,
+            (_, SourceKind::Registry) => return Ordering::Greater,
+
+            (SourceKind::LocalRegistry, SourceKind::LocalRegistry) => {}
+            (SourceKind::LocalRegistry, _) => return Ordering::Less,
+            (_, SourceKind::LocalRegistry) => return Ordering::Greater,
+
+            (SourceKind::Directory, SourceKind::Directory) => {}
+            (SourceKind::Directory, _) => return Ordering::Less,
+            (_, SourceKind::Directory) => return Ordering::Greater,
+
+            (SourceKind::Git(a), SourceKind::Git(b)) => {
+                use GitReference::*;
+                let ord = match (a, b) {
+                    (Tag(a), Tag(b)) => a.cmp(b),
+                    (Tag(_), _) => Ordering::Less,
+                    (_, Tag(_)) => Ordering::Greater,
+
+                    (Rev(a), Rev(b)) => a.cmp(b),
+                    (Rev(_), _) => Ordering::Less,
+                    (_, Rev(_)) => Ordering::Greater,
+
+                    // See module comments in src/cargo/sources/git/utils.rs
+                    // for why `DefaultBranch` is treated specially here.
+                    (Branch(a), DefaultBranch) => a.as_str().cmp("master"),
+                    (DefaultBranch, Branch(b)) => "master".cmp(b),
+                    (Branch(a), Branch(b)) => a.cmp(b),
+                    (DefaultBranch, DefaultBranch) => Ordering::Equal,
+                };
+                if ord != Ordering::Equal {
+                    return ord;
+                }
+            }
+        }
+
+        // If the `kind` and the `url` are equal, then for git sources we also
+        // ensure that the canonical urls are equal.
+        match (&self.inner.kind, &other.inner.kind) {
+            (SourceKind::Git(_), SourceKind::Git(_)) => {
+                self.inner.canonical_url.cmp(&other.inner.canonical_url)
+            }
+            _ => self.inner.url.cmp(&other.inner.url),
+        }
     }
 }
 
@@ -387,7 +445,7 @@ impl ser::Serialize for SourceId {
         if self.is_path() {
             None::<String>.serialize(s)
         } else {
-            s.collect_str(&self.into_url())
+            s.collect_str(&self.as_url())
         }
     }
 }
@@ -439,60 +497,46 @@ impl fmt::Display for SourceId {
     }
 }
 
-// Custom equality defined as canonical URL equality for git sources and
-// URL equality for other sources, ignoring the `precise` and `name` fields.
-impl PartialEq for SourceId {
-    fn eq(&self, other: &SourceId) -> bool {
-        if ptr::eq(self.inner, other.inner) {
-            return true;
-        }
-        if self.inner.kind != other.inner.kind {
-            return false;
-        }
-        if self.inner.url == other.inner.url {
-            return true;
-        }
-
-        match (&self.inner.kind, &other.inner.kind) {
-            (SourceKind::Git(ref1), SourceKind::Git(ref2)) => {
-                ref1 == ref2 && self.inner.canonical_url == other.inner.canonical_url
-            }
-            _ => false,
-        }
-    }
-}
-
-impl PartialOrd for SourceIdInner {
-    fn partial_cmp(&self, other: &SourceIdInner) -> Option<Ordering> {
-        Some(self.cmp(other))
-    }
-}
-
-impl Ord for SourceIdInner {
-    fn cmp(&self, other: &SourceIdInner) -> Ordering {
-        match self.kind.cmp(&other.kind) {
-            Ordering::Equal => {}
-            ord => return ord,
-        }
-        match self.url.cmp(&other.url) {
-            Ordering::Equal => {}
-            ord => return ord,
-        }
-        match (&self.kind, &other.kind) {
-            (SourceKind::Git(ref1), SourceKind::Git(ref2)) => {
-                (ref1, &self.canonical_url).cmp(&(ref2, &other.canonical_url))
-            }
-            _ => self.kind.cmp(&other.kind),
-        }
-    }
-}
-
 // The hash of SourceId is used in the name of some Cargo folders, so shouldn't
 // vary. `as_str` gives the serialisation of a url (which has a spec) and so
 // insulates against possible changes in how the url crate does hashing.
+//
+// Note that the semi-funky hashing here is done to handle `DefaultBranch`
+// hashing the same as `"master"`, and also to hash the same as previous
+// versions of Cargo while it's somewhat convenient to do so (that way all
+// versions of Cargo use the same checkout).
 impl Hash for SourceId {
     fn hash<S: hash::Hasher>(&self, into: &mut S) {
-        self.inner.kind.hash(into);
+        match &self.inner.kind {
+            SourceKind::Git(GitReference::Tag(a)) => {
+                0usize.hash(into);
+                0usize.hash(into);
+                a.hash(into);
+            }
+            SourceKind::Git(GitReference::Branch(a)) => {
+                0usize.hash(into);
+                1usize.hash(into);
+                a.hash(into);
+            }
+            // For now hash `DefaultBranch` the same way as `Branch("master")`,
+            // and for more details see module comments in
+            // src/cargo/sources/git/utils.rs for why `DefaultBranch`
+            SourceKind::Git(GitReference::DefaultBranch) => {
+                0usize.hash(into);
+                1usize.hash(into);
+                "master".hash(into);
+            }
+            SourceKind::Git(GitReference::Rev(a)) => {
+                0usize.hash(into);
+                2usize.hash(into);
+                a.hash(into);
+            }
+
+            SourceKind::Path => 1usize.hash(into),
+            SourceKind::Registry => 2usize.hash(into),
+            SourceKind::LocalRegistry => 3usize.hash(into),
+            SourceKind::Directory => 4usize.hash(into),
+        }
         match self.inner.kind {
             SourceKind::Git(_) => self.inner.canonical_url.hash(into),
             _ => self.inner.url.as_str().hash(into),
@@ -501,11 +545,11 @@ impl Hash for SourceId {
 }
 
 /// A `Display`able view into a `SourceId` that will write it as a url
-pub struct SourceIdIntoUrl<'a> {
+pub struct SourceIdAsUrl<'a> {
     inner: &'a SourceIdInner,
 }
 
-impl<'a> fmt::Display for SourceIdIntoUrl<'a> {
+impl<'a> fmt::Display for SourceIdAsUrl<'a> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match *self.inner {
             SourceIdInner {
@@ -549,10 +593,10 @@ impl<'a> fmt::Display for SourceIdIntoUrl<'a> {
 
 impl GitReference {
     /// Returns a `Display`able view of this git reference, or None if using
-    /// the head of the "master" branch
+    /// the head of the default branch
     pub fn pretty_ref(&self) -> Option<PrettyRef<'_>> {
-        match *self {
-            GitReference::Branch(ref s) if *s == "master" => None,
+        match self {
+            GitReference::DefaultBranch => None,
             _ => Some(PrettyRef { inner: self }),
         }
     }
@@ -569,6 +613,7 @@ impl<'a> fmt::Display for PrettyRef<'a> {
             GitReference::Branch(ref b) => write!(f, "branch={}", b),
             GitReference::Tag(ref s) => write!(f, "tag={}", s),
             GitReference::Rev(ref s) => write!(f, "rev={}", s),
+            GitReference::DefaultBranch => unreachable!(),
         }
     }
 }
@@ -581,11 +626,11 @@ mod tests {
     #[test]
     fn github_sources_equal() {
         let loc = "https://github.com/foo/bar".into_url().unwrap();
-        let master = SourceKind::Git(GitReference::Branch("master".to_string()));
-        let s1 = SourceId::new(master.clone(), loc).unwrap();
+        let default = SourceKind::Git(GitReference::DefaultBranch);
+        let s1 = SourceId::new(default.clone(), loc).unwrap();
 
         let loc = "git://github.com/foo/bar".into_url().unwrap();
-        let s2 = SourceId::new(master, loc.clone()).unwrap();
+        let s2 = SourceId::new(default, loc.clone()).unwrap();
 
         assert_eq!(s1, s2);
 
