@@ -1,3 +1,111 @@
+//! Utilities for handling git repositories, mainly around
+//! authentication/cloning.
+//!
+//! # `DefaultBranch` vs `Branch("master")`
+//!
+//! Long ago in a repository not so far away, an author (*cough* me *cough*)
+//! didn't understand how branches work in Git. This led the author to
+//! interpret these two dependency declarations the exact same way with the
+//! former literally internally desugaring to the latter:
+//!
+//! ```toml
+//! [dependencies]
+//! foo = { git = "https://example.org/foo" }
+//! foo = { git = "https://example.org/foo", branch = "master" }
+//! ```
+//!
+//! It turns out there's this things called `HEAD` in git remotes which points
+//! to the "main branch" of a repository, and the main branch is not always
+//! literally called master. What Cargo would like to do is to differentiate
+//! these two dependency directives, with the first meaning "depend on `HEAD`".
+//!
+//! Unfortunately implementing this is a breaking change. This was first
+//! attempted in #8364 but resulted in #8468 which has two independent bugs
+//! listed on that issue. Despite this breakage we would still like to roll out
+//! this change in Cargo, but we're now going to take it very slow and try to
+//! break as few people as possible along the way. These comments are intended
+//! to log the current progress and what wonkiness you might see within Cargo
+//! when handling `DefaultBranch` vs `Branch("master")`
+//!
+//! ### Repositories with `master` and a default branch
+//!
+//! This is one of the most obvious sources of breakage. If our `foo` example
+//! in above had two branches, one called `master` and another which was
+//! actually the main branch, then Cargo's change will always be a breaking
+//! change. This is because what's downloaded is an entirely different branch
+//! if we change the meaning of the dependency directive.
+//!
+//! It's expected this is quite rare, but to handle this case nonetheless when
+//! Cargo fetches from a git remote and the dependency specification is
+//! `DefaultBranch` then it will issue a warning if the `HEAD` reference
+//! doesn't match `master`. It's expected in this situation that authors will
+//! fix builds locally by specifying `branch = 'master'`.
+//!
+//! ### Differences in `cargo vendor` configuration
+//!
+//! When executing `cargo vendor` it will print out configuration which can
+//! then be used to configure Cargo to use the `vendor` directory. Historically
+//! this configuration looked like:
+//!
+//! ```toml
+//! [source."https://example.org/foo"]
+//! git = "https://example.org/foo"
+//! branch = "master"
+//! replace-with = "vendored-sources"
+//! ```
+//!
+//! We would like to, however, transition this to not include the `branch =
+//! "master"` unless the dependency directive actually mentions a branch.
+//! Conveniently older Cargo implementations all interpret a missing `branch`
+//! as `branch = "master"` so it's a backwards-compatible change to remove the
+//! `branch = "master"` directive. As a result, `cargo vendor` will no longer
+//! emit a `branch` if the git reference is `DefaultBranch`
+//!
+//! ### Differences in lock file formats
+//!
+//! Another issue pointed out in #8364 was that `Cargo.lock` files were no
+//! longer compatible on stable and nightly with each other. The underlying
+//! issue is that Cargo was serializing `branch = "master"` *differently* on
+//! nightly than it was on stable. Historical implementations of Cargo would
+//! encode `DefaultBranch` and `Branch("master")` the same way in `Cargo.lock`,
+//! so when reading a lock file we have no way of differentiating between the
+//! two.
+//!
+//! To handle this difference in encoding of `Cargo.lock` we'll be employing
+//! the standard scheme to change `Cargo.lock`:
+//!
+//! * Add support in Cargo for a future format, don't turn it on.
+//! * Wait a long time
+//! * Turn on the future format
+//!
+//! Here the "future format" is `branch=master` shows up if you have a `branch`
+//! in `Cargo.toml`, and otherwise nothing shows up in URLs. Due to the effect
+//! on crate graph resolution, however, this flows into the next point..
+//!
+//! ### Unification in the Cargo dependency graph
+//!
+//! Today dependencies with `branch = "master"` will unify with dependencies
+//! that say nothing. (that's because the latter simply desugars). This means
+//! the two `foo` directives above will resolve to the same dependency.
+//!
+//! The best idea I've got to fix this is to basically get everyone (if anyone)
+//! to stop doing this today. The crate graph resolver will start to warn if it
+//! detects that multiple `Cargo.toml` directives are detected and mixed.  The
+//! thinking is that when we turn on the new lock file format it'll also be
+//! hard breaking change for any project which still has dependencies to
+//! both the `master` branch and not.
+//!
+//! ### What we're doing today
+//!
+//! The general goal of Cargo today is to internally distinguish
+//! `DefaultBranch` and `Branch("master")`, but for the time being they should
+//! be functionally equivalent in terms of builds. The hope is that we'll let
+//! all these warnings and such bake for a good long time, and eventually we'll
+//! flip some switches if your build has no warnings it'll work before and
+//! after.
+//!
+//! That's the dream at least, we'll see how this plays out.
+
 use crate::core::GitReference;
 use crate::util::errors::{CargoResult, CargoResultExt};
 use crate::util::paths;
@@ -74,7 +182,7 @@ impl GitRemote {
     }
 
     pub fn rev_for(&self, path: &Path, reference: &GitReference) -> CargoResult<git2::Oid> {
-        reference.resolve(&self.db_at(path)?.repo)
+        reference.resolve(&self.db_at(path)?.repo, None)
     }
 
     pub fn checkout(
@@ -99,7 +207,7 @@ impl GitRemote {
                     }
                 }
                 None => {
-                    if let Ok(rev) = reference.resolve(&db.repo) {
+                    if let Ok(rev) = reference.resolve(&db.repo, Some((&self.url, cargo_config))) {
                         return Ok((db, rev));
                     }
                 }
@@ -118,7 +226,7 @@ impl GitRemote {
             .context(format!("failed to clone into: {}", into.display()))?;
         let rev = match locked_rev {
             Some(rev) => rev,
-            None => reference.resolve(&repo)?,
+            None => reference.resolve(&repo, Some((&self.url, cargo_config)))?,
         };
 
         Ok((
@@ -150,7 +258,7 @@ impl GitDatabase {
     ) -> CargoResult<GitCheckout<'_>> {
         let mut checkout = None;
         if let Ok(repo) = git2::Repository::open(dest) {
-            let mut co = GitCheckout::new(dest, self, rev.clone(), repo);
+            let mut co = GitCheckout::new(dest, self, rev, repo);
             if !co.is_fresh() {
                 // After a successful fetch operation the subsequent reset can
                 // fail sometimes for corrupt repositories where the fetch
@@ -187,13 +295,21 @@ impl GitDatabase {
         self.repo.revparse_single(&oid.to_string()).is_ok()
     }
 
-    pub fn resolve(&self, r: &GitReference) -> CargoResult<git2::Oid> {
-        r.resolve(&self.repo)
+    pub fn resolve(
+        &self,
+        r: &GitReference,
+        remote_and_config: Option<(&Url, &Config)>,
+    ) -> CargoResult<git2::Oid> {
+        r.resolve(&self.repo, remote_and_config)
     }
 }
 
 impl GitReference {
-    pub fn resolve(&self, repo: &git2::Repository) -> CargoResult<git2::Oid> {
+    pub fn resolve(
+        &self,
+        repo: &git2::Repository,
+        remote_and_config: Option<(&Url, &Config)>,
+    ) -> CargoResult<git2::Oid> {
         let id = match self {
             // Note that we resolve the named tag here in sync with where it's
             // fetched into via `fetch` below.
@@ -217,6 +333,41 @@ impl GitReference {
                     .target()
                     .ok_or_else(|| anyhow::format_err!("branch `{}` did not have a target", s))?
             }
+
+            // See the module docs for why we're using `master` here.
+            GitReference::DefaultBranch => {
+                let master = repo
+                    .find_branch("origin/master", git2::BranchType::Remote)
+                    .chain_err(|| "failed to find branch `master`")?;
+                let master = master
+                    .get()
+                    .target()
+                    .ok_or_else(|| anyhow::format_err!("branch `master` did not have a target"))?;
+
+                if let Some((remote, config)) = remote_and_config {
+                    let head_id = repo.refname_to_id("refs/remotes/origin/HEAD")?;
+                    let head = repo.find_object(head_id, None)?;
+                    let head = head.peel(ObjectType::Commit)?.id();
+
+                    if head != master {
+                        config.shell().warn(&format!(
+                            "\
+                                fetching `master` branch from `{}` but the `HEAD` \
+                                reference for this repository is not the \
+                                `master` branch. This behavior will change \
+                                in Cargo in the future and your build may \
+                                break, so it's recommended to place \
+                                `branch = \"master\"` in Cargo.toml when \
+                                depending on this git repository to ensure \
+                                that your build will continue to work.\
+                            ",
+                            remote,
+                        ))?;
+                    }
+                }
+                master
+            }
+
             GitReference::Rev(s) => {
                 let obj = repo.revparse_single(s)?;
                 match obj.as_tag() {
@@ -320,6 +471,12 @@ impl<'a> GitCheckout<'a> {
         let ok_file = self.location.join(".cargo-ok");
         let _ = paths::remove_file(&ok_file);
         info!("reset {} to {}", self.repo.path().display(), self.revision);
+
+        // Ensure libgit2 won't mess with newlines when we vendor.
+        if let Ok(mut git_config) = self.repo.config() {
+            git_config.set_bool("core.autocrlf", false)?;
+        }
+
         let object = self.repo.find_object(self.revision, None)?;
         reset(&self.repo, &object, config)?;
         paths::create(ok_file)?;
@@ -598,7 +755,7 @@ where
                 msg.push_str(attempt);
             }
         }
-        msg.push_str("\n");
+        msg.push('\n');
         if !ssh_agent_attempts.is_empty() {
             let names = ssh_agent_attempts
                 .iter()
@@ -742,11 +899,18 @@ pub fn fetch(
             refspecs.push(format!("refs/tags/{0}:refs/remotes/origin/tags/{0}", t));
         }
 
+        GitReference::DefaultBranch => {
+            // See the module docs for why we're fetching `master` here.
+            refspecs.push(String::from("refs/heads/master:refs/remotes/origin/master"));
+            refspecs.push(String::from("HEAD:refs/remotes/origin/HEAD"));
+        }
+
         // For `rev` dependencies we don't know what the rev will point to. To
         // handle this situation we fetch all branches and tags, and then we
         // pray it's somewhere in there.
         GitReference::Rev(_) => {
-            refspecs.push(format!("refs/heads/*:refs/remotes/origin/*"));
+            refspecs.push(String::from("refs/heads/*:refs/remotes/origin/*"));
+            refspecs.push(String::from("HEAD:refs/remotes/origin/HEAD"));
             tags = true;
         }
     }
@@ -789,7 +953,8 @@ pub fn fetch(
             };
             debug!("fetch failed: {}", err);
 
-            if !repo_reinitialized && err.class() == git2::ErrorClass::Reference {
+            if !repo_reinitialized && matches!(err.class(), ErrorClass::Reference | ErrorClass::Odb)
+            {
                 repo_reinitialized = true;
                 debug!(
                     "looks like this is a corrupt repository, reinitializing \
@@ -965,6 +1130,7 @@ fn github_up_to_date(
     let github_branch_name = match reference {
         GitReference::Branch(branch) => branch,
         GitReference::Tag(tag) => tag,
+        GitReference::DefaultBranch => "HEAD",
         GitReference::Rev(_) => {
             debug!("can't use github fast path with `rev`");
             return Ok(false);
@@ -988,11 +1154,7 @@ fn github_up_to_date(
 
     // Trim off the `.git` from the repository, if present, since that's
     // optional for GitHub and won't work when we try to use the API as well.
-    let repository = if repository.ends_with(".git") {
-        &repository[..repository.len() - 4]
-    } else {
-        repository
-    };
+    let repository = repository.strip_suffix(".git").unwrap_or(repository);
 
     let url = format!(
         "https://api.github.com/repos/{}/{}/commits/{}",
@@ -1005,7 +1167,10 @@ fn github_up_to_date(
     handle.useragent("cargo")?;
     let mut headers = List::new();
     headers.append("Accept: application/vnd.github.3.sha")?;
-    headers.append(&format!("If-None-Match: \"{}\"", reference.resolve(repo)?))?;
+    headers.append(&format!(
+        "If-None-Match: \"{}\"",
+        reference.resolve(repo, None)?
+    ))?;
     handle.http_headers(headers)?;
     handle.perform()?;
     Ok(handle.response_code()? == 304)
