@@ -12,15 +12,15 @@ fn depend_on_alt_registry() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "0.0.1"
-            registry = "alternative"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "alternative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -61,15 +61,15 @@ fn depend_on_alt_registry_depends_on_same_registry_no_index() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "0.0.1"
-            registry = "alternative"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "alternative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -103,15 +103,15 @@ fn depend_on_alt_registry_depends_on_same_registry() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "0.0.1"
-            registry = "alternative"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "alternative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -145,15 +145,15 @@ fn depend_on_alt_registry_depends_on_crates_io() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "0.0.1"
-            registry = "alternative"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "alternative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -191,15 +191,15 @@ fn registry_and_path_dep_works() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            path = "bar"
-            registry = "alternative"
-        "#,
+                [dependencies.bar]
+                path = "bar"
+                registry = "alternative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .file("bar/Cargo.toml", &basic_manifest("bar", "0.0.1"))
@@ -225,15 +225,15 @@ fn registry_incompatible_with_git() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            git = ""
-            registry = "alternative"
-        "#,
+                [dependencies.bar]
+                git = ""
+                registry = "alternative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -255,23 +255,23 @@ fn cannot_publish_to_crates_io_with_registry_dependency() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-            [dependencies.bar]
-            version = "0.0.1"
-            registry = "alternative"
-        "#,
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "alternative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .file(
             ".cargo/config",
             &format!(
                 r#"
-            [registries.fakeio]
-            index = "{}"
-        "#,
+                    [registries.fakeio]
+                    index = "{}"
+                "#,
                 fakeio_url
             ),
         )
@@ -285,8 +285,8 @@ fn cannot_publish_to_crates_io_with_registry_dependency() {
         .file(
             "config.json",
             r#"
-            {"dl": "https://crates.io/api/v1/crates", "api": "https://crates.io"}
-        "#,
+                {"dl": "https://crates.io/api/v1/crates", "api": "https://crates.io"}
+            "#,
         )
         .build();
 
@@ -311,15 +311,15 @@ fn publish_with_registry_dependency() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "0.0.1"
-            registry = "alternative"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "alternative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -359,6 +359,8 @@ fn publish_with_registry_dependency() {
             "readme": null,
             "readme_file": null,
             "repository": null,
+            "homepage": null,
+            "documentation": null,
             "vers": "0.0.1"
         }"#,
         "foo-0.0.1.crate",
@@ -372,18 +374,18 @@ fn alt_registry_and_crates_io_deps() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            crates_io_dep = "0.0.1"
+                [dependencies]
+                crates_io_dep = "0.0.1"
 
-            [dependencies.alt_reg_dep]
-            version = "0.1.0"
-            registry = "alternative"
-        "#,
+                [dependencies.alt_reg_dep]
+                version = "0.1.0"
+                registry = "alternative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -459,6 +461,8 @@ fn publish_to_alt_registry() {
             "readme": null,
             "readme_file": null,
             "repository": null,
+            "homepage": null,
+            "documentation": null,
             "vers": "0.0.1"
         }"#,
         "foo-0.0.1.crate",
@@ -472,16 +476,16 @@ fn publish_with_crates_io_dep() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = ["me"]
-            license = "MIT"
-            description = "foo"
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = ["me"]
+                license = "MIT"
+                description = "foo"
 
-            [dependencies.bar]
-            version = "0.0.1"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -522,6 +526,8 @@ fn publish_with_crates_io_dep() {
             "readme": null,
             "readme_file": null,
             "repository": null,
+            "homepage": null,
+            "documentation": null,
             "vers": "0.0.1"
         }"#,
         "foo-0.0.1.crate",
@@ -566,16 +572,16 @@ fn patch_alt_reg() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
+                [package]
+                name = "foo"
+                version = "0.0.1"
 
-            [dependencies]
-            bar = { version = "0.1.0", registry = "alternative" }
+                [dependencies]
+                bar = { version = "0.1.0", registry = "alternative" }
 
-            [patch.alternative]
-            bar = { path = "bar" }
-        "#,
+                [patch.alternative]
+                bar = { path = "bar" }
+            "#,
         )
         .file(
             "src/lib.rs",
@@ -606,15 +612,15 @@ fn bad_registry_name() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "0.0.1"
-            registry = "bad name"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "bad name"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -637,7 +643,7 @@ Caused by:
         "owner",
         "publish",
         "search",
-        "yank",
+        "yank --vers 0.0.1",
     ] {
         p.cargo(cmd)
             .arg("--registry")
@@ -667,14 +673,14 @@ fn no_api() {
         .file(
             "Cargo.toml",
             r#"
-             [package]
-             name = "foo"
-             version = "0.0.1"
+                [package]
+                name = "foo"
+                version = "0.0.1"
 
-             [dependencies.bar]
-             version = "0.0.1"
-             registry = "alternative"
-         "#,
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "alternative"
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -737,14 +743,14 @@ fn alt_reg_metadata() {
         .file(
             "Cargo.toml",
             r#"
-             [package]
-             name = "foo"
-             version = "0.0.1"
+                [package]
+                name = "foo"
+                version = "0.0.1"
 
-             [dependencies]
-             altdep = { version = "0.0.1", registry = "alternative" }
-             iodep = { version = "0.0.1" }
-         "#,
+                [dependencies]
+                altdep = { version = "0.0.1", registry = "alternative" }
+                iodep = { version = "0.0.1" }
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -814,6 +820,8 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
+                        "homepage": null,
+                        "documentation": null,
                         "edition": "2015",
                         "links": null
                     }
@@ -836,28 +844,6 @@ fn alt_reg_metadata() {
             r#"
              {
                 "packages": [
-                    {
-                        "name": "altdep2",
-                        "version": "0.0.1",
-                        "id": "altdep2 0.0.1 (registry+file:[..]/alternative-registry)",
-                        "license": null,
-                        "license_file": null,
-                        "description": null,
-                        "source": "registry+file:[..]/alternative-registry",
-                        "dependencies": [],
-                        "targets": "{...}",
-                        "features": {},
-                        "manifest_path": "[..]/altdep2-0.0.1/Cargo.toml",
-                        "metadata": null,
-                        "publish": null,
-                        "authors": [],
-                        "categories": [],
-                        "keywords": [],
-                        "readme": null,
-                        "repository": null,
-                        "edition": "2015",
-                        "links": null
-                    },
                     {
                         "name": "altdep",
                         "version": "0.0.1",
@@ -890,6 +876,56 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
+                        "homepage": null,
+                        "documentation": null,
+                        "edition": "2015",
+                        "links": null
+                    },
+                    {
+                        "name": "altdep2",
+                        "version": "0.0.1",
+                        "id": "altdep2 0.0.1 (registry+file:[..]/alternative-registry)",
+                        "license": null,
+                        "license_file": null,
+                        "description": null,
+                        "source": "registry+file:[..]/alternative-registry",
+                        "dependencies": [],
+                        "targets": "{...}",
+                        "features": {},
+                        "manifest_path": "[..]/altdep2-0.0.1/Cargo.toml",
+                        "metadata": null,
+                        "publish": null,
+                        "authors": [],
+                        "categories": [],
+                        "keywords": [],
+                        "readme": null,
+                        "repository": null,
+                        "homepage": null,
+                        "documentation": null,
+                        "edition": "2015",
+                        "links": null
+                    },
+                    {
+                        "name": "bar",
+                        "version": "0.0.1",
+                        "id": "bar 0.0.1 (registry+https://github.com/rust-lang/crates.io-index)",
+                        "license": null,
+                        "license_file": null,
+                        "description": null,
+                        "source": "registry+https://github.com/rust-lang/crates.io-index",
+                        "dependencies": [],
+                        "targets": "{...}",
+                        "features": {},
+                        "manifest_path": "[..]/bar-0.0.1/Cargo.toml",
+                        "metadata": null,
+                        "publish": null,
+                        "authors": [],
+                        "categories": [],
+                        "keywords": [],
+                        "readme": null,
+                        "repository": null,
+                        "homepage": null,
+                        "documentation": null,
                         "edition": "2015",
                         "links": null
                     },
@@ -937,6 +973,8 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
+                        "homepage": null,
+                        "documentation": null,
                         "edition": "2015",
                         "links": null
                     },
@@ -972,28 +1010,8 @@ fn alt_reg_metadata() {
                         "keywords": [],
                         "readme": null,
                         "repository": null,
-                        "edition": "2015",
-                        "links": null
-                    },
-                    {
-                        "name": "bar",
-                        "version": "0.0.1",
-                        "id": "bar 0.0.1 (registry+https://github.com/rust-lang/crates.io-index)",
-                        "license": null,
-                        "license_file": null,
-                        "description": null,
-                        "source": "registry+https://github.com/rust-lang/crates.io-index",
-                        "dependencies": [],
-                        "targets": "{...}",
-                        "features": {},
-                        "manifest_path": "[..]/bar-0.0.1/Cargo.toml",
-                        "metadata": null,
-                        "publish": null,
-                        "authors": [],
-                        "categories": [],
-                        "keywords": [],
-                        "readme": null,
-                        "repository": null,
+                        "homepage": null,
+                        "documentation": null,
                         "edition": "2015",
                         "links": null
                     }
@@ -1019,14 +1037,14 @@ fn unknown_registry() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "0.0.1"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -1055,6 +1073,43 @@ fn unknown_registry() {
             r#"
             {
               "packages": [
+                {
+                  "name": "bar",
+                  "version": "0.0.1",
+                  "id": "bar 0.0.1 (registry+https://github.com/rust-lang/crates.io-index)",
+                  "license": null,
+                  "license_file": null,
+                  "description": null,
+                  "source": "registry+https://github.com/rust-lang/crates.io-index",
+                  "dependencies": [
+                    {
+                      "name": "baz",
+                      "source": "registry+file://[..]/alternative-registry",
+                      "req": "^0.0.1",
+                      "kind": null,
+                      "rename": null,
+                      "optional": false,
+                      "uses_default_features": true,
+                      "features": [],
+                      "target": null,
+                      "registry": "file:[..]/alternative-registry"
+                    }
+                  ],
+                  "targets": "{...}",
+                  "features": {},
+                  "manifest_path": "[..]",
+                  "metadata": null,
+                  "publish": null,
+                  "authors": [],
+                  "categories": [],
+                  "keywords": [],
+                  "readme": null,
+                  "repository": null,
+                  "homepage": null,
+                  "documentation": null,
+                  "edition": "2015",
+                  "links": null
+                },
                 {
                   "name": "baz",
                   "version": "0.0.1",
@@ -1074,6 +1129,8 @@ fn unknown_registry() {
                   "keywords": [],
                   "readme": null,
                   "repository": null,
+                  "homepage": null,
+                  "documentation": null,
                   "edition": "2015",
                   "links": null
                 },
@@ -1109,41 +1166,8 @@ fn unknown_registry() {
                   "keywords": [],
                   "readme": null,
                   "repository": null,
-                  "edition": "2015",
-                  "links": null
-                },
-                {
-                  "name": "bar",
-                  "version": "0.0.1",
-                  "id": "bar 0.0.1 (registry+https://github.com/rust-lang/crates.io-index)",
-                  "license": null,
-                  "license_file": null,
-                  "description": null,
-                  "source": "registry+https://github.com/rust-lang/crates.io-index",
-                  "dependencies": [
-                    {
-                      "name": "baz",
-                      "source": "registry+file://[..]/alternative-registry",
-                      "req": "^0.0.1",
-                      "kind": null,
-                      "rename": null,
-                      "optional": false,
-                      "uses_default_features": true,
-                      "features": [],
-                      "target": null,
-                      "registry": "file:[..]/alternative-registry"
-                    }
-                  ],
-                  "targets": "{...}",
-                  "features": {},
-                  "manifest_path": "[..]",
-                  "metadata": null,
-                  "publish": null,
-                  "authors": [],
-                  "categories": [],
-                  "keywords": [],
-                  "readme": null,
-                  "repository": null,
+                  "homepage": null,
+                  "documentation": null,
                   "edition": "2015",
                   "links": null
                 }
@@ -1181,15 +1205,15 @@ fn registries_index_relative_url() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "0.0.1"
-            registry = "relative"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "relative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -1230,15 +1254,15 @@ fn registries_index_relative_path_not_allowed() {
         .file(
             "Cargo.toml",
             r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies.bar]
-            version = "0.0.1"
-            registry = "relative"
-        "#,
+                [dependencies.bar]
+                version = "0.0.1"
+                registry = "relative"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
