@@ -78,6 +78,17 @@ fn list_commands_with_descriptions() {
         .run();
 }
 
+#[cargo_test]
+fn list_aliases_with_descriptions() {
+    let p = project().build();
+    p.cargo("--list")
+        .with_stdout_contains("    b                    alias: build")
+        .with_stdout_contains("    c                    alias: check")
+        .with_stdout_contains("    r                    alias: run")
+        .with_stdout_contains("    t                    alias: test")
+        .run();
+}
+
 #[cargo_test]
 fn list_command_looks_at_path() {
     let proj = project().build();
@@ -151,10 +162,10 @@ error: no such subcommand: `biuld`
         .file(
             "src/main.rs",
             r#"
-            fn main() {
-                println!("Similar, but not identical to, build");
-            }
-        "#,
+                fn main() {
+                    println!("Similar, but not identical to, build");
+                }
+            "#,
         )
         .publish();
 
@@ -303,11 +314,11 @@ fn cargo_subcommand_args() {
         .file(
             "src/main.rs",
             r#"
-            fn main() {
-                let args: Vec<_> = ::std::env::args().collect();
-                println!("{:?}", args);
-            }
-        "#,
+                fn main() {
+                    let args: Vec<_> = ::std::env::args().collect();
+                    println!("{:?}", args);
+                }
+            "#,
         )
         .build();
 
@@ -327,35 +338,6 @@ fn cargo_subcommand_args() {
         .run();
 }
 
-#[cargo_test]
-fn cargo_help() {
-    cargo_process("").run();
-    cargo_process("help").run();
-    cargo_process("-h").run();
-    cargo_process("help build").run();
-    cargo_process("build -h").run();
-    cargo_process("help help").run();
-}
-
-#[cargo_test]
-fn cargo_help_external_subcommand() {
-    Package::new("cargo-fake-help", "1.0.0")
-        .file(
-            "src/main.rs",
-            r#"
-            fn main() {
-                if ::std::env::args().nth(2) == Some(String::from("--help")) {
-                    println!("fancy help output");
-                }
-            }"#,
-        )
-        .publish();
-    cargo_process("install cargo-fake-help").run();
-    cargo_process("help fake-help")
-        .with_stdout("fancy help output\n")
-        .run();
-}
-
 #[cargo_test]
 fn explain() {
     cargo_process("--explain E0001")
@@ -365,15 +347,6 @@ fn explain() {
         .run();
 }
 
-// Test that the output of `cargo -Z help` shows a different help screen with
-// all the `-Z` flags.
-#[cargo_test]
-fn z_flags_help() {
-    cargo_process("-Z help")
-        .with_stdout_contains("    -Z unstable-options -- Allow the usage of unstable options")
-        .run();
-}
-
 #[cargo_test]
 fn closed_output_ok() {
     // Checks that closed output doesn't cause an error.
