@@ -82,9 +82,9 @@ fn simple_git_ignore_exists() {
         contents,
         "/target\n\
          **/some.file\n\n\
-         #Added by cargo\n\
+         # Added by cargo\n\
          #\n\
-         #already existing elements were commented out\n\
+         # already existing elements were commented out\n\
          \n\
          #/target\n\
          Cargo.lock\n",
@@ -108,7 +108,7 @@ fn git_ignore_exists_no_conflicting_entries() {
     assert_eq!(
         contents,
         "**/some.file\n\n\
-         #Added by cargo\n\
+         # Added by cargo\n\
          \n\
          /target\n\
          Cargo.lock\n",
@@ -305,7 +305,8 @@ fn invalid_dir_name() {
         .with_stderr(
             "\
 [ERROR] invalid character `.` in crate name: `foo.bar`, [..]
-use --name to override crate name",
+If you need a crate name to not match the directory name, consider using --name flag.
+",
         )
         .run();
 
@@ -323,7 +324,7 @@ fn reserved_name() {
         .with_stderr(
             "\
 [ERROR] the name `test` cannot be used as a crate name, it conflicts [..]\n\
-use --name to override crate name
+If you need a crate name to not match the directory name, consider using --name flag.
 ",
         )
         .run();
