@@ -335,14 +335,16 @@ fn change_features_rebuilds() {
     Package::new("foo", "1.0.0")
         .file(
             "src/main.rs",
-            r#"fn main() {
-            if cfg!(feature = "f1") {
-                println!("f1");
+            r#"
+            fn main() {
+                if cfg!(feature = "f1") {
+                    println!("f1");
+                }
+                if cfg!(feature = "f2") {
+                    println!("f2");
+                }
             }
-            if cfg!(feature = "f2") {
-                println!("f2");
-            }
-        }"#,
+            "#,
         )
         .file(
             "Cargo.toml",
@@ -799,11 +801,9 @@ fn already_installed_updates_yank_status_on_upgrade() {
 
     cargo_process("install foo --version=1.0.1")
         .with_status(101)
-        .with_stderr(
-            "\
-[UPDATING] `[..]` index
-[ERROR] could not find `foo` in registry `[..]` with version `=1.0.1`
-",
+        .with_stderr_contains(
+            "error: cannot install package `foo`, it has been yanked from registry \
+            `https://github.com/rust-lang/crates.io-index`",
         )
         .run();
 
