@@ -13,11 +13,11 @@ fn metabuild_gated() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            metabuild = ["mb"]
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                metabuild = ["mb"]
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -43,16 +43,16 @@ fn basic_project() -> Project {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            metabuild = ["mb", "mb-other"]
+                cargo-features = ["metabuild"]
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                metabuild = ["mb", "mb-other"]
 
-            [build-dependencies]
-            mb = {path="mb"}
-            mb-other = {path="mb-other"}
-        "#,
+                [build-dependencies]
+                mb = {path="mb"}
+                mb-other = {path="mb-other"}
+            "#,
         )
         .file("src/lib.rs", "")
         .file("mb/Cargo.toml", &basic_lib_manifest("mb"))
@@ -63,10 +63,10 @@ fn basic_project() -> Project {
         .file(
             "mb-other/Cargo.toml",
             r#"
-            [package]
-            name = "mb-other"
-            version = "0.0.1"
-        "#,
+                [package]
+                name = "mb-other"
+                version = "0.0.1"
+            "#,
         )
         .file(
             "mb-other/src/lib.rs",
@@ -91,15 +91,15 @@ fn metabuild_error_both() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            metabuild = "mb"
+                cargo-features = ["metabuild"]
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                metabuild = "mb"
 
-            [build-dependencies]
-            mb = {path="mb"}
-        "#,
+                [build-dependencies]
+                mb = {path="mb"}
+            "#,
         )
         .file("src/lib.rs", "")
         .file("build.rs", r#"fn main() {}"#)
@@ -130,12 +130,12 @@ fn metabuild_missing_dep() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            metabuild = "mb"
-        "#,
+                cargo-features = ["metabuild"]
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                metabuild = "mb"
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -159,15 +159,15 @@ fn metabuild_optional_dep() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            metabuild = "mb"
+                cargo-features = ["metabuild"]
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                metabuild = "mb"
 
-            [build-dependencies]
-            mb = {path="mb", optional=true}
-        "#,
+                [build-dependencies]
+                mb = {path="mb", optional=true}
+            "#,
         )
         .file("src/lib.rs", "")
         .file("mb/Cargo.toml", &basic_lib_manifest("mb"))
@@ -195,26 +195,26 @@ fn metabuild_lib_name() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            metabuild = "mb"
+                cargo-features = ["metabuild"]
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                metabuild = "mb"
 
-            [build-dependencies]
-            mb = {path="mb"}
-        "#,
+                [build-dependencies]
+                mb = {path="mb"}
+            "#,
         )
         .file("src/lib.rs", "")
         .file(
             "mb/Cargo.toml",
             r#"
-            [package]
-            name = "mb"
-            version = "0.0.1"
-            [lib]
-            name = "other"
-        "#,
+                [package]
+                name = "mb"
+                version = "0.0.1"
+                [lib]
+                name = "other"
+            "#,
         )
         .file(
             "mb/src/lib.rs",
@@ -243,15 +243,15 @@ fn metabuild_fresh() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            metabuild = "mb"
+                cargo-features = ["metabuild"]
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                metabuild = "mb"
 
-            [build-dependencies]
-            mb = {path="mb"}
-        "#,
+                [build-dependencies]
+                mb = {path="mb"}
+            "#,
         )
         .file("src/lib.rs", "")
         .file("mb/Cargo.toml", &basic_lib_manifest("mb"))
@@ -285,26 +285,28 @@ fn metabuild_links() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            links = "cat"
-            metabuild = "mb"
+                cargo-features = ["metabuild"]
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                links = "cat"
+                metabuild = "mb"
 
-            [build-dependencies]
-            mb = {path="mb"}
-        "#,
+                [build-dependencies]
+                mb = {path="mb"}
+            "#,
         )
         .file("src/lib.rs", "")
         .file("mb/Cargo.toml", &basic_lib_manifest("mb"))
         .file(
             "mb/src/lib.rs",
-            r#"pub fn metabuild() {
-                assert_eq!(std::env::var("CARGO_MANIFEST_LINKS"),
-                    Ok("cat".to_string()));
-                println!("Hello mb");
-            }"#,
+            r#"
+                pub fn metabuild() {
+                    assert_eq!(std::env::var("CARGO_MANIFEST_LINKS"),
+                        Ok("cat".to_string()));
+                    println!("Hello mb");
+                }
+            "#,
         )
         .build();
 
@@ -320,16 +322,16 @@ fn metabuild_override() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            links = "cat"
-            metabuild = "mb"
+                cargo-features = ["metabuild"]
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                links = "cat"
+                metabuild = "mb"
 
-            [build-dependencies]
-            mb = {path="mb"}
-        "#,
+                [build-dependencies]
+                mb = {path="mb"}
+            "#,
         )
         .file("src/lib.rs", "")
         .file("mb/Cargo.toml", &basic_lib_manifest("mb"))
@@ -341,9 +343,9 @@ fn metabuild_override() {
             ".cargo/config",
             &format!(
                 r#"
-            [target.{}.cat]
-            rustc-link-lib = ["a"]
-        "#,
+                    [target.{}.cat]
+                    rustc-link-lib = ["a"]
+                "#,
                 rustc_host()
             ),
         )
@@ -358,37 +360,37 @@ fn metabuild_workspace() {
         .file(
             "Cargo.toml",
             r#"
-            [workspace]
-            members = ["member1", "member2"]
-        "#,
+                [workspace]
+                members = ["member1", "member2"]
+            "#,
         )
         .file(
             "member1/Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "member1"
-            version = "0.0.1"
-            metabuild = ["mb1", "mb2"]
+                cargo-features = ["metabuild"]
+                [package]
+                name = "member1"
+                version = "0.0.1"
+                metabuild = ["mb1", "mb2"]
 
-            [build-dependencies]
-            mb1 = {path="../../mb1"}
-            mb2 = {path="../../mb2"}
-        "#,
+                [build-dependencies]
+                mb1 = {path="../../mb1"}
+                mb2 = {path="../../mb2"}
+            "#,
         )
         .file("member1/src/lib.rs", "")
         .file(
             "member2/Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "member2"
-            version = "0.0.1"
-            metabuild = ["mb1"]
+                cargo-features = ["metabuild"]
+                [package]
+                name = "member2"
+                version = "0.0.1"
+                metabuild = ["mb1"]
 
-            [build-dependencies]
-            mb1 = {path="../../mb1"}
-        "#,
+                [build-dependencies]
+                mb1 = {path="../../mb1"}
+            "#,
         )
         .file("member2/src/lib.rs", "")
         .build();
@@ -454,95 +456,95 @@ fn metabuild_build_plan() {
         .masquerade_as_nightly_cargo()
         .with_json(
             r#"
-{
-    "invocations": [
-        {
-            "package_name": "mb",
-            "package_version": "0.5.0",
-            "target_kind": ["lib"],
-            "compile_mode": "build",
-            "kind": null,
-            "deps": [],
-            "outputs": [
-                "[..]/target/debug/deps/libmb-[..].rlib",
-                "[..]/target/debug/deps/libmb-[..].rmeta"
-            ],
-            "links": {},
-            "program": "rustc",
-            "args": "{...}",
-            "env": "{...}",
-            "cwd": "[..]"
-        },
-        {
-            "package_name": "mb-other",
-            "package_version": "0.0.1",
-            "target_kind": ["lib"],
-            "compile_mode": "build",
-            "kind": null,
-            "deps": [],
-            "outputs": [
-                "[..]/target/debug/deps/libmb_other-[..].rlib",
-                "[..]/target/debug/deps/libmb_other-[..].rmeta"
-            ],
-            "links": {},
-            "program": "rustc",
-            "args": "{...}",
-            "env": "{...}",
-            "cwd": "[..]"
-        },
-        {
-            "package_name": "foo",
-            "package_version": "0.0.1",
-            "target_kind": ["custom-build"],
-            "compile_mode": "build",
-            "kind": null,
-            "deps": [0, 1],
-            "outputs": "{...}",
-            "links": "{...}",
-            "program": "rustc",
-            "args": "{...}",
-            "env": "{...}",
-            "cwd": "[..]"
-        },
-        {
-            "package_name": "foo",
-            "package_version": "0.0.1",
-            "target_kind": ["custom-build"],
-            "compile_mode": "run-custom-build",
-            "kind": null,
-            "deps": [2],
-            "outputs": [],
-            "links": {},
-            "program": "[..]/foo/target/debug/build/foo-[..]/metabuild-foo",
-            "args": [],
-            "env": "{...}",
-            "cwd": "[..]"
-        },
-        {
-            "package_name": "foo",
-            "package_version": "0.0.1",
-            "target_kind": ["lib"],
-            "compile_mode": "build",
-            "kind": null,
-            "deps": [3],
-            "outputs": [
-                "[..]/foo/target/debug/deps/libfoo-[..].rlib",
-                "[..]/foo/target/debug/deps/libfoo-[..].rmeta"
-            ],
-            "links": "{...}",
-            "program": "rustc",
-            "args": "{...}",
-            "env": "{...}",
-            "cwd": "[..]"
-        }
-    ],
-    "inputs": [
-        "[..]/foo/Cargo.toml",
-        "[..]/foo/mb/Cargo.toml",
-        "[..]/foo/mb-other/Cargo.toml"
-    ]
-}
-"#,
+            {
+                "invocations": [
+                    {
+                        "package_name": "mb",
+                        "package_version": "0.5.0",
+                        "target_kind": ["lib"],
+                        "compile_mode": "build",
+                        "kind": null,
+                        "deps": [],
+                        "outputs": [
+                            "[..]/target/debug/deps/libmb-[..].rlib",
+                            "[..]/target/debug/deps/libmb-[..].rmeta"
+                        ],
+                        "links": {},
+                        "program": "rustc",
+                        "args": "{...}",
+                        "env": "{...}",
+                        "cwd": "[..]"
+                    },
+                    {
+                        "package_name": "mb-other",
+                        "package_version": "0.0.1",
+                        "target_kind": ["lib"],
+                        "compile_mode": "build",
+                        "kind": null,
+                        "deps": [],
+                        "outputs": [
+                            "[..]/target/debug/deps/libmb_other-[..].rlib",
+                            "[..]/target/debug/deps/libmb_other-[..].rmeta"
+                        ],
+                        "links": {},
+                        "program": "rustc",
+                        "args": "{...}",
+                        "env": "{...}",
+                        "cwd": "[..]"
+                    },
+                    {
+                        "package_name": "foo",
+                        "package_version": "0.0.1",
+                        "target_kind": ["custom-build"],
+                        "compile_mode": "build",
+                        "kind": null,
+                        "deps": [0, 1],
+                        "outputs": "{...}",
+                        "links": "{...}",
+                        "program": "rustc",
+                        "args": "{...}",
+                        "env": "{...}",
+                        "cwd": "[..]"
+                    },
+                    {
+                        "package_name": "foo",
+                        "package_version": "0.0.1",
+                        "target_kind": ["custom-build"],
+                        "compile_mode": "run-custom-build",
+                        "kind": null,
+                        "deps": [2],
+                        "outputs": [],
+                        "links": {},
+                        "program": "[..]/foo/target/debug/build/foo-[..]/metabuild-foo",
+                        "args": [],
+                        "env": "{...}",
+                        "cwd": "[..]"
+                    },
+                    {
+                        "package_name": "foo",
+                        "package_version": "0.0.1",
+                        "target_kind": ["lib"],
+                        "compile_mode": "build",
+                        "kind": null,
+                        "deps": [3],
+                        "outputs": [
+                            "[..]/foo/target/debug/deps/libfoo-[..].rlib",
+                            "[..]/foo/target/debug/deps/libfoo-[..].rmeta"
+                        ],
+                        "links": "{...}",
+                        "program": "rustc",
+                        "args": "{...}",
+                        "env": "{...}",
+                        "cwd": "[..]"
+                    }
+                ],
+                "inputs": [
+                    "[..]/foo/Cargo.toml",
+                    "[..]/foo/mb/Cargo.toml",
+                    "[..]/foo/mb-other/Cargo.toml"
+                ]
+            }
+            "#,
         )
         .run();
 
@@ -557,36 +559,36 @@ fn metabuild_two_versions() {
         .file(
             "Cargo.toml",
             r#"
-            [workspace]
-            members = ["member1", "member2"]
-        "#,
+                [workspace]
+                members = ["member1", "member2"]
+            "#,
         )
         .file(
             "member1/Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "member1"
-            version = "0.0.1"
-            metabuild = ["mb"]
+                cargo-features = ["metabuild"]
+                [package]
+                name = "member1"
+                version = "0.0.1"
+                metabuild = ["mb"]
 
-            [build-dependencies]
-            mb = {path="../../mb1"}
-        "#,
+                [build-dependencies]
+                mb = {path="../../mb1"}
+            "#,
         )
         .file("member1/src/lib.rs", "")
         .file(
             "member2/Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "member2"
-            version = "0.0.1"
-            metabuild = ["mb"]
+                cargo-features = ["metabuild"]
+                [package]
+                name = "member2"
+                version = "0.0.1"
+                metabuild = ["mb"]
 
-            [build-dependencies]
-            mb = {path="../../mb2"}
-        "#,
+                [build-dependencies]
+                mb = {path="../../mb2"}
+            "#,
         )
         .file("member2/src/lib.rs", "")
         .build();
@@ -640,15 +642,15 @@ fn metabuild_external_dependency() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["metabuild"]
-            [package]
-            name = "dep"
-            version = "1.0.0"
-            metabuild = ["mb"]
+                cargo-features = ["metabuild"]
+                [package]
+                name = "dep"
+                version = "1.0.0"
+                metabuild = ["mb"]
 
-            [build-dependencies]
-            mb = "1.0"
-        "#,
+                [build-dependencies]
+                mb = "1.0"
+            "#,
         )
         .file("src/lib.rs", "")
         .build_dep("mb", "1.0.0")
@@ -683,38 +685,40 @@ fn metabuild_json_artifact() {
         .masquerade_as_nightly_cargo()
         .with_json_contains_unordered(
             r#"
-{
-  "executable": null,
-  "features": [],
-  "filenames": "{...}",
-  "fresh": false,
-  "package_id": "foo [..]",
-  "profile": "{...}",
-  "reason": "compiler-artifact",
-  "target": {
-    "crate_types": [
-      "bin"
-    ],
-    "doctest": false,
-    "edition": "2018",
-    "kind": [
-      "custom-build"
-    ],
-    "name": "metabuild-foo",
-    "src_path": "[..]/foo/target/.metabuild/metabuild-foo-[..].rs"
-  }
-}
+            {
+              "executable": null,
+              "features": [],
+              "filenames": "{...}",
+              "fresh": false,
+              "package_id": "foo [..]",
+              "profile": "{...}",
+              "reason": "compiler-artifact",
+              "target": {
+                "crate_types": [
+                  "bin"
+                ],
+                "doc": false,
+                "doctest": false,
+                "edition": "2018",
+                "kind": [
+                  "custom-build"
+                ],
+                "name": "metabuild-foo",
+                "src_path": "[..]/foo/target/.metabuild/metabuild-foo-[..].rs",
+                "test": false
+              }
+            }
 
-{
-  "cfgs": [],
-  "env": [],
-  "linked_libs": [],
-  "linked_paths": [],
-  "package_id": "foo [..]",
-  "out_dir": "[..]",
-  "reason": "build-script-executed"
-}
-"#,
+            {
+              "cfgs": [],
+              "env": [],
+              "linked_libs": [],
+              "linked_paths": [],
+              "package_id": "foo [..]",
+              "out_dir": "[..]",
+              "reason": "build-script-executed"
+            }
+            "#,
         )
         .run();
 }
@@ -729,31 +733,33 @@ fn metabuild_failed_build_json() {
         .with_status(101)
         .with_json_contains_unordered(
             r#"
-{
-  "message": {
-    "children": "{...}",
-    "code": "{...}",
-    "level": "error",
-    "message": "cannot find function `metabuild` in [..] `mb`",
-    "rendered": "[..]",
-    "spans": "{...}"
-  },
-  "package_id": "foo [..]",
-  "reason": "compiler-message",
-  "target": {
-    "crate_types": [
-      "bin"
-    ],
-    "doctest": false,
-    "edition": "2018",
-    "kind": [
-      "custom-build"
-    ],
-    "name": "metabuild-foo",
-    "src_path": null
-  }
-}
-"#,
+            {
+              "message": {
+                "children": "{...}",
+                "code": "{...}",
+                "level": "error",
+                "message": "cannot find function `metabuild` in [..] `mb`",
+                "rendered": "[..]",
+                "spans": "{...}"
+              },
+              "package_id": "foo [..]",
+              "reason": "compiler-message",
+              "target": {
+                "crate_types": [
+                  "bin"
+                ],
+                "doc": false,
+                "doctest": false,
+                "edition": "2018",
+                "kind": [
+                  "custom-build"
+                ],
+                "name": "metabuild-foo",
+                "src_path": null,
+                "test": false
+              }
+            }
+            "#,
         )
         .run();
 }
