@@ -8,16 +8,16 @@ fn inherits_on_release() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [profile.release]
-            inherits = "dev"
-        "#,
+                [profile.release]
+                inherits = "dev"
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -27,10 +27,7 @@ fn inherits_on_release() {
         .with_status(101)
         .with_stderr(
             "\
-[ERROR] failed to parse manifest at [..]
-
-Caused by:
-  An 'inherits' must not specified root profile 'release'
+[ERROR] `inherits` must not be specified in root profile `release`
 ",
         )
         .run();
@@ -42,16 +39,16 @@ fn missing_inherits() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [profile.release-lto]
-            codegen-units = 7
-        "#,
+                [profile.release-lto]
+                codegen-units = 7
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -61,10 +58,9 @@ fn missing_inherits() {
         .with_status(101)
         .with_stderr(
             "\
-[ERROR] failed to parse manifest at [..]
-
-Caused by:
-  An 'inherits' directive is needed for all profiles that are not 'dev' or 'release'. Here it is missing from 'release-lto'",
+[ERROR] profile `release-lto` is missing an `inherits` directive \
+    (`inherits` is required for all profiles except `dev` or `release`)
+",
         )
         .run();
 }
@@ -75,17 +71,17 @@ fn invalid_profile_name() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [profile.'.release-lto']
-            inherits = "release"
-            codegen-units = 7
-        "#,
+                [profile.'.release-lto']
+                inherits = "release"
+                codegen-units = 7
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -109,18 +105,18 @@ fn invalid_dir_name() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [profile.'release-lto']
-            inherits = "release"
-            dir-name = ".subdir"
-            codegen-units = 7
-        "#,
+                [profile.'release-lto']
+                inherits = "release"
+                dir-name = ".subdir"
+                codegen-units = 7
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -144,17 +140,17 @@ fn invalid_inherits() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [profile.'release-lto']
-            inherits = ".release"
-            codegen-units = 7
-        "#,
+                [profile.'release-lto']
+                inherits = ".release"
+                codegen-units = 7
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -178,17 +174,17 @@ fn non_existent_inherits() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [profile.release-lto]
-            codegen-units = 7
-            inherits = "non-existent"
-        "#,
+                [profile.release-lto]
+                codegen-units = 7
+                inherits = "non-existent"
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -198,10 +194,8 @@ fn non_existent_inherits() {
         .with_status(101)
         .with_stderr(
             "\
-[ERROR] failed to parse manifest at [..]
-
-Caused by:
-  Profile 'non-existent' not found in Cargo.toml",
+[ERROR] profile `release-lto` inherits from `non-existent`, but that profile is not defined
+",
         )
         .run();
 }
@@ -212,17 +206,17 @@ fn self_inherits() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [profile.release-lto]
-            codegen-units = 7
-            inherits = "release-lto"
-        "#,
+                [profile.release-lto]
+                codegen-units = 7
+                inherits = "release-lto"
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -232,10 +226,8 @@ fn self_inherits() {
         .with_status(101)
         .with_stderr(
             "\
-[ERROR] failed to parse manifest at [..]
-
-Caused by:
-  Inheritance loop of profiles cycles with profile 'release-lto'",
+[ERROR] profile inheritance loop detected with profile `release-lto` inheriting `release-lto`
+",
         )
         .run();
 }
@@ -246,21 +238,21 @@ fn inherits_loop() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [profile.release-lto]
-            codegen-units = 7
-            inherits = "release-lto2"
+                [profile.release-lto]
+                codegen-units = 7
+                inherits = "release-lto2"
 
-            [profile.release-lto2]
-            codegen-units = 7
-            inherits = "release-lto"
-        "#,
+                [profile.release-lto2]
+                codegen-units = 7
+                inherits = "release-lto"
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -270,10 +262,8 @@ fn inherits_loop() {
         .with_status(101)
         .with_stderr(
             "\
-[ERROR] failed to parse manifest at [..]
-
-Caused by:
-  Inheritance loop of profiles cycles with profile 'release-lto'",
+[ERROR] profile inheritance loop detected with profile `release-lto2` inheriting `release-lto`
+",
         )
         .run();
 }
@@ -284,32 +274,32 @@ fn overrides_with_custom() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [dependencies]
-            xxx = {path = "xxx"}
-            yyy = {path = "yyy"}
+                [dependencies]
+                xxx = {path = "xxx"}
+                yyy = {path = "yyy"}
 
-            [profile.dev]
-            codegen-units = 7
+                [profile.dev]
+                codegen-units = 7
 
-            [profile.dev.package.xxx]
-            codegen-units = 5
-            [profile.dev.package.yyy]
-            codegen-units = 3
+                [profile.dev.package.xxx]
+                codegen-units = 5
+                [profile.dev.package.yyy]
+                codegen-units = 3
 
-            [profile.other]
-            inherits = "dev"
-            codegen-units = 2
+                [profile.other]
+                inherits = "dev"
+                codegen-units = 2
 
-            [profile.other.package.yyy]
-            codegen-units = 6
-        "#,
+                [profile.other.package.yyy]
+                codegen-units = 6
+            "#,
         )
         .file("src/lib.rs", "")
         .file("xxx/Cargo.toml", &basic_lib_manifest("xxx"))
@@ -358,11 +348,11 @@ fn conflicting_usage() {
         .file(
             "Cargo.toml",
             r#"
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-        "#,
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+            "#,
         )
         .file("src/lib.rs", "")
         .build();
@@ -386,16 +376,16 @@ fn clean_custom_dirname() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["named-profiles"]
+                cargo-features = ["named-profiles"]
 
-            [package]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
+                [package]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
 
-            [profile.other]
-            inherits = "release"
-        "#,
+                [profile.other]
+                inherits = "release"
+            "#,
         )
         .file("src/main.rs", "fn main() {}")
         .build();
@@ -477,3 +467,32 @@ fn clean_custom_dirname() {
     assert!(p.build_dir().join("debug").is_dir());
     assert!(!p.build_dir().join("other").is_dir());
 }
+
+#[cargo_test]
+fn unknown_profile() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            cargo-features = ["named-profiles"]
+
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("build --profile alpha -Zunstable-options")
+        .masquerade_as_nightly_cargo()
+        .with_stderr("[ERROR] profile `alpha` is not defined")
+        .with_status(101)
+        .run();
+    // Clean has a separate code path, need to check it too.
+    p.cargo("clean --profile alpha -Zunstable-options")
+        .masquerade_as_nightly_cargo()
+        .with_stderr("[ERROR] profile `alpha` is not defined")
+        .with_status(101)
+        .run();
+}
